clear
close all

cd D:\Kopriva\MATLAB\work\HrZZ_2013_2015\Nonlinear_MS_experiments

% J_Chemometrics_vol_29_no_11_pp_615_626_2015
%%     
dataset = 3;           % 1 - peptide synthesis from June 2013: 9 mixtures,
                       %     25 pure components
                       % 2 - ????? from March/April 2014: 12 mixtures, 19
                       %           pure components
                       % 3 - ????  from May 22 2014: 12 mixtures, 28 PC
                       
                       
kernel = 'gaussian';   % Options: 'polynomial', 'gaussian'; 'exponential'
                       % 'sigmoid', 'laplace', 'multiquad', 'invmultiquad'
                     
fact_type = 0;         % 1 - NMU; 2 - FastNMF (for A) and T_SBL for S (in mapped space)


lin_fact_type = 1;   % for EKM and EFM based nonlinear algorithms
                       % 1 - NMU; 2 - NMF_L0,3 - ONP-MF 4 - FastSepNMF and Smooth L0
                       % 5 - JADE, 6 - SOBI, 7 - EFICA,
                       % 8 - kernel ICA, 9 - TDSEP 

correction = 0;        %  0/1 flag: used to correct bias (DC)term introduced by polynomial, exponential and Gaussian kernels
                       %  0 - no correction; 1 - correction. 
                       
                     
d = 8;                % order of approximate EFM
b = 1;                 % for appoximate polynomial EFM
    
dp = 10;              % degree of poly kernel
bp = 1e0;             % offset of the poly kernel
sig2 = 1.0e0;         % variance of Gaussian kernel, exponential or Laplacian kernels
sig = 1e7;            % standard deviation for sigmoid kernel
c = 1e2;              % offset for sigmoid, multiquadratic and inverse multiquadratic kernels

D = 500;              % subspace dimension for EKM
 
preproc = 3;           % 0 - NO preprocessing; 
                       % 1 - hard thresholding with threshold tau
                       % 2 - soft thresholding with threshold tau
                       % 3 - low-rank decomposition with regularization
                       %     constant lambda
                       % 4 - trimmed threshodling with alpha=3.5
tau = 1e-5;
alpha = 3.5;
lam = 0.05; 

if dataset == 1
    M = 25;              % number of components
elseif dataset == 2
    M = 19;
elseif dataset == 3
    M = 28;
end

K = 5;               % number of overlapping sources

if ( strcmp(kernel,'polynomial'))
    par1=dp; par2 = bp;
elseif ( strcmp(kernel,'gaussian'))
    par1=sig2; par2=[];
elseif ( strcmp(kernel,'exponential'))
    par1=sig2; par2=[];
elseif ( strcmp(kernel,'sigmoid'))
    par1 = sig; par2 = c;
elseif ( strcmp(kernel,'laplace'))
    par1 = sig2; par2=[];
elseif ( strcmp(kernel,'multiquad'))
    par1 = c; par2=[];
elseif ( strcmp(kernel,'invmultiquad'))
    par1 = c; par2=[];
end

%% load experimental MS data

[X, S] = load_mass_spectra(dataset);

% take last time chromatogram
x = X(size(X,1),:);
clear X

%%
xm = max(max(x));
X = x/xm;

%% preprocessing of original data (if any...)
if preproc == 1
    Y=(sign(X-tau)+1)/2;
    X=X.*Y;
elseif preproc == 2
    X = max(0,X-tau);
elseif preproc == 3
    X = proximal_gradient_rpca(X, lam);
elseif preproc == 4
    X = max(0,X.*(power(X,alpha)-power(tau,alpha))./power(X,alpha));
end

%% use feature map to generate multichannel mixture
%X = feature_map(X,'gaussian',sqrt(sig2),d);
X = feature_map(X,'Vandermonde',d,[]);

%% preprocessing of original data (if any...)
if preproc == 1
    Y=(sign(X-tau)+1)/2;
    X=X.*Y;
elseif preproc == 2
    X = max(0,X-tau);
elseif preproc == 3
    X = proximal_gradient_rpca(X, lam);
elseif preproc == 4
    X = max(0,X.*(power(X,alpha)-power(tau,alpha))./power(X,alpha));
end

    %% find D-dimensional basis by k-means clustering
    % eliminate small components
    Xn=sqrt(sum(X.*X));
    [xmax indmax] = max(Xn);
    Xn = Xn./xmax;
    P=Xn-1/100000; P=sign(P); P=(-P+1)/2;
    Xnp=Xn.*P;
    [xd,Ind]=setdiff(Xn,Xnp);
    %XR = X(:,Ind);
    XR = X;
    
    cnum=1e5;
    if (D <= 20)
         epscnum=9.0;
    else
         epscnum=1e6;
    end
    
    while cnum > epscnum
         [SH,B]=kmeans(XR',D,'distance','sqEuclidean','Replicates',1,'EmptyAction','singleton'); %,'start','sample');
        V = B';
        cnum=cond(V)
    end
    clear SH 
    
    V = X;
    X = empirical_kernel_map(X,V,kernel,correction,par1,par2);
    NSOURCE = size(X,1);
     
%% preprocessing of mapped data (if any...)
if preproc == 1
    Y=(sign(X-tau)+1)/2;
    X=X.*Y;
elseif preproc == 2
    X = max(0,X-tau);
elseif preproc == 4
    X = max(0,X.*(power(X,alpha)-power(tau,alpha))./power(X,alpha));
end

%% Sparseness constrained factorization
% NMU
ts = tic;
NSOURCE = 8500;
SH = linear_factorization(X,lin_fact_type,NSOURCE,14);

time_PT_EKM_NMU = toc(ts);

NSOURCE = size(SH,1);

for i=1:NSOURCE
    for j=1:M
        crpcsh(i,j)=SH(i,:)*S(j,:)'/norm(SH(i,:))/norm(S(j,:));
    end
end
 
for j=1:M
    [PT_EKM_NMU_crmax(j) PT_EKM_NMU_crind(j)]=max(crpcsh(:,j));
end

ind=PT_EKM_NMU_crind;
S_PC = SH(ind,:);

if dataset == 1
    cd .\Results
elseif dataset == 2
    cd .\Results
elseif dataset == 3
    cd .\Results
end

%save S_PC_NMU_TT_alpha_3p5_M_D_9901.mat S_PC

%save EKM_NMU_TT_M_D_9901_crmax.txt EKM_SCA_crmax -ascii
%save EJM_NMU_TT_M_D_9901_crind.txt EKM_SCA_crind -ascii

save S_PC_SCH_EFM_EKM_RPCA_NMU_D_9901.mat S_PC

save SCH_EFM_EKM_RPCA_NMU_D_9901_crmax.txt PT_EKM_NMU_crmax -ascii
save SCH_EFM_EKM_RPCA_NMU_D_9901_crind.txt PT_EKM_NMU_crind -ascii

display('PT_EKM_NMU_crmax')
PT_EKM_NMU_crmax
display('PT_EKM_NMU_crind')
PT_EKM_NMU_crind
display('PT_EKM_NMU_time')
time_PT_EKM_NMU
