%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% Copyright (c) by Ivica Kopriva, IRB, Croatia; 
% April 2013.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

function [SH]= linear_factorization(X,alg,NSOURCE,K)

% INPUTS:
% X : NxT data matrix to be factorized
% 
% alg       % 1 - NMU; 2 - NMF_L0, 3 - ONP-MF
            % 4 - FastSepNMF and Smooth L0
            % 5 - JADE, 6 - SOBI, 7 - EFICA,
            % 8 - kernel ICA, 9 - TDSEP, 10 - SD + APU 

% NSOURCE - number of sources
% K - nummber of overlapping sources
%
% OUTPUTS:
%
% SH - NSOURCExT matrix of extracted sources
%

if(alg == 1)
        xmin = min(min(X));
        if xmin < 0
            display('Negative entried are not allowed by NMU algorithm')
            display('They will be set to zero')
             X = max(X,0);
        end
        norm_type = 2;
        [St,A] = recursive_NMU(X',NSOURCE,norm_type,40);
        SH = St';
elseif (alg == 2)
        xmin = min(min(X));
        if xmin < 0
            display('Negative entried are not allowed by NMF_L0 algorithm')
            display('They will be set to zero')
            X = max(X,0);
        end
  
        %% set NMFL0_H parameters
        options.K = NSOURCE;
        options.L = K;    % number of overlapping sources
        options.numIter = 7;

        %%% select sparse coder
        options.sparseCoder = @rsNNLS___;
        % options.sparseCoder = @sNNLS___;

        %%% select dictionary update method
        options.updateType = 'ANLS_FC';
        % options.updateType = 'MU';
        % options.updateType = 'NNKSVD';
        % options.updateType = 'ANLS_PG';
        % options.NNLS_PG_tolerance = 1e-3;
        % options.NNLS_PG_maxIter = 100;

        %%% number of dictionary updates
        options.numUpdateIter = 10;
        [AH,SH,INFO] = NMFL0_H(X,options);
elseif (alg == 3)
    maxOnpmfIters = 3000;
    X(X<0) = 0;
    % ONPMF
    [AH,SH,relError,actualIters] = onpmf(X,NSOURCE,maxOnpmfIters);
elseif (alg == 4)
        xmin = min(min(X));
        if xmin < 0
            display('Negative entried are not allowed by Smooth L0 algorithm')
            display('They will be set to zero')
            X = max(X,0);
        end
        
       % [J,normM,U] = FastSepNMF(X,NSOURCE,0);
       % J = FastConicalHull(X,NSOURCE);
        %   AH=X(:,J);
        [E,C]=EIA_WM(X);           % A lattice matrix method for hyperspec image unmixing (Information scieneces vol. 181 pp. 1787-1803, 2011).
        [I3 NSOURCE] = size(E);    
        NSOURCE = NSOURCE-1;
        for n=1:NSOURCE
            AH(:,n)=E(:,n)/norm(E(:,n));
        end

        sigma_off = 0.005;
        sigma_decrease_factor = 0.95;
        L = 1;
        if sigma_off>0
           sigma_min = sigma_off*4;
        else
           sigma_min = 0.0001;
        end
        % Global Init:
        mu_0 = 0.1;
        SH=SL0_MMV(AH,X,sigma_min,sigma_decrease_factor,mu_0,L,pinv(AH));     
elseif (alg == 5)
        W = jadeR(X,NSOURCE);
        SH = W*X;
elseif (alg == 6)
        [A, SH] = sobi(X,NSOURCE,4); 
elseif (alg == 7)
        %[A, W] = fastica(X,'numOfIC', NSOURCE, 'g', 'pow3', 'displayMode', 'off');
        [W, SIRefica, Wsymm, SIRsymm]=efica(X);
        SH = W*X;
elseif (alg == 8)
        W=kernel_ica(X);
        SH = W*X;
elseif (alg == 9)
        [SH,W] = tdsep3(X,0:3);
elseif (alg == 10)
        [ind, SH_sd] = sd_somp(X, NSOURCE);
        SH = apu(X, X(:,ind)); 
end